/*****************************************************************************
 *
 *  ParamDialog.cc                              XMM Science Analysis Subsystem
 *
 *  Parameter Dialog for the SAS GUI
 *
 *  (c) 1998  ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: ParamDialog.cc,v 1.23 2004/02/25 14:14:35 aibarra Exp $
 *
 *****************************************************************************/

// enable for debugging
#undef DEBUG_PARAMDIALOG

// Application headers
#include "ParamInfoSet.h"            // Application headers
#include "ParamDialog.h"
#include "ParamWidget.h"
#include "LYTReader.h"
#include "GuiBuilder.h"
#include "ValEdit.h"
#ifdef DEBUG_PARAMDIALOG
#  include "LayoutPrinter.h"
#endif
#include "LayoutParFile.h"
#include <SasFileLocator.h>

// C++ headers
#include <cassert>
#include <iostream>           
#include <sstream>
#include <fstream>
//#include <algo.h>
#include <map>

// Qt headers
#include <qdialog.h>
#include <qpushbutton.h>
#include <qtooltip.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qframe.h>
#include <qgroupbox.h>
#include <qlayout.h>
#include <qstring.h>
#include <qapplication.h>
//Included to deal with the warning message if you click on the Save button
#include <qmessagebox.h>

//================================================================
//  Class ParamDialog
//================================================================

//----------------------------
//  Constructors & destructors
//----------------------------

/*** Constructor ***/

ParamDialog::ParamDialog(ParameterInfo* info,
			 QWidget *parent,
			 const char *name,
			 bool useTabs,             // Use tabs if no layout file
			 const char* okButton,     // Label on 'Ok' button
			 const char* lytFileName)  // Name of layout file
 : QDialog(parent, name, true), _canvas(0)
{
    assert(info);   
    setCaption(info->taskName().c_str());

    
    // Copy the ParameterInfo so we can revert
        
    _originalInfo = info;
    _info = new ParameterInfo(*info);

    /* Create a canvas with a scrollable view. The canvas is embedded
       in a frame to provide a border which scrolls with the canvas. */
    QVBoxLayout* vbox = new QVBoxLayout(this);
    



    /* Add parameter widgets to the canvas
       Find out the longest parameter name (pixels), and pass it on */
    bool hasParameters=false;
    QSize labelSize(0,0);
    {
      list<string> names=_info->names();
      hasParameters=!names.empty();
      for ( list<string>::const_iterator it=names.begin();
	    it!=names.end();it++) {
	//QSize s(QLabel(QString((*it).c_str()),0).sizeHint());
	QString qstr((*it).c_str());
	QSize s(QLabel(qstr,0).sizeHint());
	if (s.width() > labelSize.width()) labelSize.setWidth(s.width());
	if (s.height() > labelSize.height()) labelSize.setHeight(s.height());
      }
    }

    // 
    _layout = createLayout(useTabs, lytFileName);
    GuiBuilder builder = GuiBuilder(this,labelSize);
    _layout->build(&builder);
    _canvas=builder.widget();

    vbox->addWidget(_canvas);
    vbox->addStretch(1);
    vbox->addSpacing(5);

#ifdef DEBUG_PARAMDIALOG
    // Print the layout with a LayoutPrinter
    cout << endl << "LayoutPrinter:" << endl;
    LayoutPrinter printer = LayoutPrinter(&cout);
    _layout->build(&printer);
#endif

    // dialog buttons ...
    QHBoxLayout* hbox=new QHBoxLayout(vbox,-1,"DialogButtons");

    QPushButton* runButton = new QPushButton(okButton, this );
    connect(runButton, SIGNAL(clicked()), SLOT(run()) );
    QToolTip::add(runButton,"Run the task");
    hbox->addStretch(1);
    hbox->addWidget(runButton);

    QPushButton* cancelButton = new QPushButton("Cancel", this);
    connect(cancelButton, SIGNAL(clicked()), SLOT(reject()) );
    QToolTip::add(cancelButton,"Close the dialog");
    hbox->addSpacing(2);
    hbox->addWidget(cancelButton);

    // ... only add these if we actually have parameters
    if (hasParameters) {
      QPushButton* saveButton = new QPushButton("Save", this );
      connect(saveButton, SIGNAL(clicked()), SLOT(save()) );
      QToolTip::add(saveButton,"Save as default values");
      hbox->addSpacing(2);
      hbox->addWidget(saveButton);

      QPushButton* defaultButton = new QPushButton("Defaults", this);
      connect(defaultButton, SIGNAL(clicked()), SLOT(revertToDefault()) );
      QToolTip::add(defaultButton,"Revert to default values");
      hbox->addSpacing(2);
      hbox->addWidget(defaultButton);
    }

    hbox->addSpacing(5);
    vbox->addSpacing(5);


    QSize ds=sizeHint();
    if (ds.width() > maxWidth) ds.setWidth(maxWidth);
    resize(ds);

}


//  Destructor **

ParamDialog::~ParamDialog()
{
    delete _canvas;     // Delete view ...
    delete _info;       // ... then model
    delete _layout;
}


//-----------------------------
//  Layout
//-----------------------------

//  Create a layout description for the parameter widgets [ PRIVATE ] **

Layout* ParamDialog::createLayout(bool useTabs, const char* lytFileName)
{
    /*
     *  NOTE: maxPerTab should be related to maximum dialog height,
     *  so that tabbed layouts don't need scroll bars.
     */

    const int maxPerTab = 10;

    Layout* layout = 0;

    SasFileLocator locator("");
    string filename(_info->taskName() + ".lyt");
    if(lytFileName)
        filename = string(lytFileName) + ".lyt";
    string lytFile = locator.locate("config/" + filename, "", false);

    // Use the layout file if it exists
    LYTReader lytReader(_info);
    if(lytFile.length() && lytReader.open(lytFile))
    {
	lytReader.parse(layout);
    }

    // or flat but separated by tabs...
    else if(useTabs && int(_info->names().size()) > maxPerTab)
        layout = createTabbedLayout(maxPerTab);

    // create one your own:
    if (!layout) {
      LayoutParFile creator(_info);
      creator.create(layout);
    }

    return layout;
}


//  Construct simple column layout for '_info' [ PRIVATE ] **

Layout* ParamDialog::createColumnLayout()
{
    ColumnLayout* column = new ColumnLayout;
    list<string> namelist = _info->names();
    list<string>::iterator it = namelist.begin();
    while(it != namelist.end())
    {
	string name = *it++;
	Parameter* p = (*_info)[name];
	if(p)
	{
	    SimpleLayout* pl = new SimpleLayout(p);
	    column->addItem(pl);
	}
    }
    return column;
}


//  Construct tabbed layout for '_info' [ PRIVATE ] **

Layout* ParamDialog::createTabbedLayout(int maxPerTab)
{
    PagedLayout* tabpad  = new PagedLayout;
    ColumnLayout* column = new ColumnLayout;

    list<string> namelist = _info->names();
    list<string>::iterator it = namelist.begin();
    int item = 1;
    int page = 0;
    while(it != namelist.end())
    {
        string name = *it++;
	Parameter* p = (*_info)[name];
	if(p)
        {
	    SimpleLayout* pl = new SimpleLayout(p);
	    column->addItem(pl);
	    
	    if(item++ == maxPerTab || it == namelist.end())
	    {
	        ostringstream ostr;
		ostr << page++;
		column->setName(ostr.str());
		tabpad->addItem(column);
		column = new ColumnLayout;
		item = 1;
	    }
	}
    }
    return tabpad;
}


//-----------------------------
//  Button actions
//-----------------------------

//  Run the task [ PRIVATE SLOT ] **

void ParamDialog::run()
{
  //  topLevelWidget()->setCaption( "New Caption" ); 
    *_originalInfo = *_info;
    accept();
}


//  Revert to default values  [ PRIVATE SLOT ] **

void ParamDialog::revertToDefault()
{
    _info->revertToDefault();
}

void ParamDialog::save()
{

  //This warning message window has been created to warn the users about the effect of 
  //creating a parameter file.
  //AI: 2004/02/25
  string taskName = _info->taskName();
  
  string warningMessage = "You are going to save these parameteres in the following preference file:\n\n"
    "$HOME/.sas.d/config/" + taskName +".par\n\n" + 
    "You have to remember that this file will be used to set\nthe default " + 
    "parameters in future sas sessions\n(or even if you run a sas task through the command line).\nYou should remove it, if you don't want that to happen.";

    switch( QMessageBox::warning(this, "Save Warning Message", 
				 warningMessage.c_str(),
				 "Ok",
				 "Cancel", 0, 0, 1 ) ) {
    case 0: // The user clicked the Retry again button or pressed Enter
      // Save parameters
      ParamInfoSet::instance()->saveParamInfo(_info);
      break;
    case 1: // The user clicked the Quit or pressed Escape
      // exit
      break;
    }
    
    



//  QMessageBox::warning();

}

//  function that does the trick *
extern "C" bool popUpDialog(ParameterInfo* info)
{
  QApplication* app=0;
  if (!qApp) {
      char** argv=0;
      int    argc=0;
      app=new QApplication(argc,argv);
  }
  bool useTabs=true;
  ParamDialog* dialog = new ParamDialog(info, 0, 0, useTabs, "Run");
  
  bool ok = (dialog->exec() != 0);
  delete dialog;

  /* Cannot delete app, as qApp is global available. Deleting it ruins
     any Qt application that follows
  */
  //delete app;

  return ok;
}
