/*****************************************************************************
 *
 *  Process.h                                   XMM Science Analysis Subsystem
 *
 *  Class for UNIX process
 *
 *  (c) 1999 ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: Process.h,v 1.3 1999/10/18 07:47:03 jbrumfit Exp $
 *
 *****************************************************************************/

#ifndef PROCESS_H
#define PROCESS_H

#include <sys/types.h>           // C headers

#include <string>                // C++ headers

#include <qobject.h>             // Qt headers
#include <qsocketnotifier.h>

#include "LineBuffer.h"          // Application headers


//================================================================
//  Class Process
//
//  An instance of Process starts and controls a system process, for
//  use within a Qt-based application.
//
//  The application receives asynchronous notification of completion
//  and output from the process, via Qt signals, so that the GUI event
//  loop can continue. The "done" signal returns the exit status of
//  the process. The output signals are line-buffered to simplify
//  interleaving of messages from different processes. The buffers
//  are flushed automatically when the process terminates.
//
//  Once the process has terminated, it is possible to reuse the 
//  Process object to run another process.
//
//  Example:
//
//     _task = new Process("foo",this);
//     connect(_task,SIGNAL(done(int)),this,SLOT(childDone(int)));
//     connect(_task,SIGNAL(outputMessage(const char*)),
//                   SLOT(outputReceived(const char*)));
//     connect(_task,SIGNAL(errorMessage(const char*)),
//                   SLOT(errorReceived(const char*)));
//
//     char *argv[] = { "foo",0 };
//     _task->execute(argv);
//
//  The 'waitfor' method should not normally be used, as it blocks the
//  Qt event loop and can cause deadlock if the processes output buffer
//  (pipe) becomes full. Instead, the caller should use asynchonous
//  notification via the "done" signal.
//
//  If a process is killed with the "kill" method, the "done" signal
//  returns an exit status of '-1'.
//
//  If the application (parent process) terminates before the child
//  process, the child process is killed by the Process's destructor.
//  If the parent is simply killed, the child will continue running
//  with the 'init' process (PID=1) as its parent.
//
//  It is also possible to send information to the process' stdin:
//
//    _task = new Process("foo",this);
//    _task->enableInput();
//    char *argv[] = { "foo",0 };
//    _task->execute(argv);
//    ...
//    _task->sendInput("hello");
//
//  Possible future enhancements:
//
//    - constructors for extra 'argv' types (e.g. QList)
//    - support sending Message objects over an additional pipe
//
//  Limitations:
//
//    - at least one output must be connected in order for the
//      "done" signal to work.
//
//================================================================


class Process : public QObject
{
    Q_OBJECT

public:

    // Type definitions
    typedef enum { NOTSTARTED, RUNNING, DONE, FAILED } Status;

    // Constructors and destructors
    Process(const string& name, QObject *parent=0);
    ~Process();

    void   execute(char** argv);
    Status childStatus();
    void   kill();
    int    waitFor();
    void   enableInput(bool enable=true);

signals:
    void done(int);
    void outputMessage(const char*);
    void errorMessage(const char*);

public slots:
    void sendInput(const char* text);

private slots:
    void outReceived(int fd);
    void errReceived(int fd);

private:
    // Slots
    void dataReceived(int fd, QSocketNotifier* notifier, LineBuffer& buffer);

    void connectNotify(const char* signal);

    // Test status
    bool childExists();

    // Member data
    string            _name;           // Name of process
    pid_t             _childPid;       // PID of child process
    Status            _status;         // Cached status
    int               _exitCode;       // Exit code of process
    int               _infd;           // File descriptor for input pipe
    int               _outfd;          // File descriptor for output pipe
    int               _errfd;          // File descriptor for error pipe
    QSocketNotifier*  _outNotifier;    // Read notifier for output pipe
    QSocketNotifier*  _errNotifier;    // Read notifier for error pipe
    int               _openPipes;      // Number of open pipes
    LineBuffer        _outBuffer;      // Buffer for output
    LineBuffer        _errBuffer;      // Buffer for errors
    bool              _inRequired;     // Child requires input
    bool              _outConnected;   // outputMessage signal is connected
    bool              _errConnected;   // errorMessage signal is connected
};

#endif

