/*****************************************************************************
 *
 *  LogPane.cc                                  XMM Science Analysis Subsystem
 *
 *  Scrolling pane to display log
 *
 *  (c) 1999  ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: LogPane.cc,v 1.5 2003/01/23 09:21:49 ulammers Exp $
 *
 *****************************************************************************/

// Application headers
#include "LogPane.h"
#include "Task.h"
#include "System.h"

// C++ headers
#include <iostream>
#include <fstream>

// Qt headers
#include <qpainter.h>
#include <qfiledialog.h>
#include <qstring.h>
#include <qmessagebox.h>
#include <qpaintdevicemetrics.h>
#include <qfontmetrics.h>
#include <qwidget.h>
#include <qprinter.h>
#include <qstatusbar.h>


//================================================================
//  Class LogPane
//================================================================

/*** Constructor ***/

LogPane::LogPane(QWidget* parent)
  : QMultiLineEdit(parent)
{
    // Configure the log pane

    setMinimumHeight(40);       // Has no effect with Qt-2.0
    setReadOnly(true);

    _logs = 0;
}


/*** Destructor ***/

LogPane::~LogPane()
{
    if(_logs)
        _logs->close();
}


/*** Enable logging to a file ***/

void LogPane::setLogFile(const string& logFileName)
{
    if(_logs)
        _logs->close();

    _logs = new ofstream(logFileName.c_str());
}


/*** Write a message to the log-pane and the log file ***/

void LogPane::writeToLog(const string& message)
{
    insertLine(message.c_str());
    setCursorPosition(numLines(),0); 

    if(_logs)
        (*_logs) << message << endl;
}


//----------------------------------------
//  Scripting
//----------------------------------------

/*** Save the log as a shell script ***/

void LogPane::saveAsScript()
{
    QString file(QFileDialog::getSaveFileName(0,0,0,"sas_script"));
    if(file.isNull())
      return;
    if(!QFileInfo(file).exists()
       || !QMessageBox::information(this,"SAS","Overwrite existing file?",
				    "Ok","Cancel"))
    {
      Info::Task* self=Info::Task::current();

      ofstream os(file.latin1());
      os << "#!/bin/sh\n#\n"
	 << "# XMM Science Analysis System\n"
	 << "# " << self->name() << " (" << self->package()
	 << '-' << self->version() << ") [" << self->package() << "]\n"
	 << "# Date: " <<  System::getDateTimeString() << "\n" << endl;
      
      int nlines = numLines();
      for(int i=0; i < nlines; i++)
	{
	  const char* s = textLine(i);
	  if(s[0] == '@' && s[1] == '@')
	    os << &s[3] << endl; // forget the space
	    //	    os << string(s).substr(2) << endl << endl;
	}
      os << endl;
      os.close();

      makeExecutable(file.latin1());
    }
}


//----------------------------------------
//  Printing
//----------------------------------------

/*** Print the log pane ***/

void LogPane::print(QPrinter* printer, QStatusBar* statusBar)
{
    const int MARGIN = 10;

    if (printer->setup(this)) {
	statusBar->message( "Printing..." );
	QPainter p;
	p.begin(printer);
	p.setFont(font());
	int yPos = 0;
	QFontMetrics fm = p.fontMetrics();
	QPaintDeviceMetrics metrics(printer);

	int pageNo = 1;
	for(int i = 0 ; i < numLines() ; i++)
	{
	    if(MARGIN + yPos > metrics.height() - MARGIN)
	    {
		QString msg;
		msg.sprintf("Printing (page %d)...", ++pageNo);
		statusBar->message(msg);
		printer->newPage();
		yPos = 0;
	    }
	    p.drawText(MARGIN, MARGIN + yPos,
		       metrics.width(), fm.lineSpacing(),
		       ExpandTabs | DontClip,
		       textLine(i));
	    yPos = yPos + fm.lineSpacing();
	}
	p.end();
	statusBar->message("Printing completed", 2000);
    } else {
	statusBar->message("Printing aborted", 2000);
    }
}
