/*****************************************************************************
 *
 *  SasTask.cc                                     XMM Science Analysis Subsystem
 *
 *  SAS SasTask control
 *
 *  (c) 1999 ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: SasTask.cc,v 1.6 2009/08/31 15:17:17 aibarra Exp $
 *
 *****************************************************************************/

#include "SasTask.h"                    // Application headers
#include "Logger.h"

#include <ParameterInfo.h>
#include <errstr.h>
#include <StringListParameter.h>
#include <AlertDialog.h>

#include <iostream>                // C++ headers
#include <vector>
#include <string>
#include <cassert>


//================================================================
//  Class SasTask
//================================================================

//----------------------------
//  Constructors & destructors
//----------------------------

/*
 *  Construct a SasTask object which can be executed later.
 *
 *  The 'info' parameters are copied, so that caller may modify or
 *  delete the info, without affecting a task that is queued or already
 *  running.
 *
 *  The task preferences are currently NOT copied, as changes to the
 *  corresponding environment variables are logged immediately,
 *  rather than when the task is run.
 */

SasTask::SasTask(ParameterInfo* info, QObject* parent, GuiPreferences* pref)
  : QObject(parent)
{
    assert(info);

    _parameters  = new ParameterInfo(*info);
    _preferences = pref;

    /*
     *  The following makes the preferences part of the closure,
     *  but requires logging environment variables when the task
     *  is executed, instead of when the preferences are set.
     */
//  if(_preferences)
//      _preferences = new GuiPreferences(*pref);
}


/*** Destructor ***/

SasTask::~SasTask()
{
    // _process is deleted automatically by parent QObject

    if(_parameters)
        delete _parameters;

//  The following is needed if the preferences are copied.
//  if(_preferences)
//      delete _preferences;
}


//----------------------------
//  SasTask control
//----------------------------

/*** Set environment for task from user preferences [ PRIVATE ] ***/

void SasTask::setupTaskEnvironment()
{
   if(!_preferences)
       return;
   /*
    _preferences->setEnvt("SAS_ODF", "ODF");
    _preferences->setEnvt("SAS_CCF", "CCF");
    _preferences->setEnvt("SAS_VERBOSITY", "verbosity");
   */
}


/*** Execute a process and catch the output ***/

void SasTask::execute()
{
    /*** Convert the argument list into an "argv" vector. ***/

    vector<string> args = _parameters->asArgumentList();

    int nargs = args.size();
    /*
     * AI gcc4 porting
     * error: array bound forbidden after parenthesized type-id
     *  note: try removing the parentheses around the type-id
     *  new (char*)[nargs + 1]; => new  char* [nargs + 1];
     */
    char** argv = new char* [nargs + 1];
    for(int i = 0; i < nargs; i++)
        argv[i] = const_cast<char*>((args[i]).c_str());
    argv[nargs] = 0;


    /*** Log a message to say that task has started ***/

    Logger::writeCommand(_parameters->asCommandLine());


    /*** Set environment variables specified in preferences ***/

    setupTaskEnvironment();


    /*** Create a process and connect output handlers ***/

    _process = new Process(name(), this);
    connect(_process,SIGNAL(done(int)),this,SLOT(processDone(int)));
    connect(_process,SIGNAL(outputMessage(const char*)),
	    SLOT(outputReceived(const char*)));
    connect(_process,SIGNAL(errorMessage(const char*)),
	    SLOT(errorReceived(const char*)));

    _process->execute(argv);

    // 'argv' can be deleted now, because the process accesses
    // a COPY created by the 'fork'.

    delete[] argv;
    argv = 0;
}


/*** Kill the task ***/

void SasTask::kill()
{
    if(_process)
        _process->kill();
}


//----------------------------
//  Message handling
//----------------------------

/*** Handle 'done' message from process [ PRIVATE SLOT ] ***/

void SasTask::processDone(int code)
{
    if(code == 127)
        errstr << "Could not find task " << name()
	       << Sas::error("nosuchtask");

    emit done(code, this);
//    removeChild(_process);
//    delete _process;
}


// Handle output from process [ PRIVATE SLOT ]
void SasTask::outputReceived(const char* message)
{

  Logger::write(message);
}


// Handle error output from process [ PRIVATE SLOT ]
void SasTask::errorReceived(const char* message)
{
  Logger::write(message);
}

