/*****************************************************************************
 *
 *  SasTask.h                                      XMM Science Analysis Subsystem
 *
 *  SAS SasTask control
 *
 *  (c) 1999 ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: SasTask.h,v 1.3 2003/01/28 10:59:35 jbakker Exp $
 *
 *****************************************************************************/

#ifndef SasTask_H
#define SasTask_H

#include <string>
#include <qobject.h>

#include "Process.h"
#include "ParameterInfo.h"
#include "GuiPreferences.h"

class ParameterInfo;

//=========================================================================
//  Class SasTask
//
//  Responsibilities:
//    - proxy for SAS task
//    - closure with parameters (which can be queued)
//    - running and killing SasTask
//    - asynchronous notification of messages
//    - asynchronous notification of completion
//
//  An instance of SasTask starts and controls an SAS task, with a given set of
//  parameters. The constructor creates a SasTask object, with a copy of the 
//  parameters, which may be queued for later execution.
//
//  Example:
//
//      task = new SasTask(paramInfo, this);
//      conect(task,SIGNAL(done(int,SasTask*)),SLOT(done(int,SasTask*)));
//      task->execute();
//
//      Demo::done() { delete task; }
//
//  The task notifies completion with a Qt signal 'done', which returns the 
//  exit code from the process(exitCode). If the SasTask is killed, the 'done'
//  signal is still generated, but with an exit code of -1.
//
//  The implementation currently forks a separate process and passes the 
//  parameters to it via the command line. An alternative implementation could
//  start a new thread and call the task as a function.
//
//==========================================================================

class SasTask : public QObject
{
    Q_OBJECT

public:
    // Constructors and destructors
    SasTask(ParameterInfo* info, QObject* parent=0, GuiPreferences* pref=0);
    ~SasTask();

    // SasTask control
    void execute();
    void kill();
  
    // Information
    // Have to ensure that returned name points to a physical, non-temporary piece
    // of memory - hence temporary variable
    const char* name() { _tstring=_parameters->taskName();return _tstring.c_str(); }

signals:
    void done(int exitCode, SasTask* task);

private slots:
    void processDone(int code);
    void outputReceived(const char*);
    void errorReceived(const char*);

private:
    string _tstring; // local variable to hold taskname as a string permanently
	//
    void setupTaskEnvironment();

    // Disable copy constructor and operator=
    SasTask(const SasTask&);
    SasTask& operator=(const SasTask&);

    // Member data
    Process*          _process;
    ParameterInfo*    _parameters;
    GuiPreferences*   _preferences;

};

#endif

